"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var express_1 = __importDefault(require("express"));
var bridge_error_1 = __importDefault(require("./bridge-error"));
var CoronadoBridge = /** @class */ (function () {
    /**
     * constructor
     * This function configures and starts CoronadoBridge.
     * 1. Instantiates the OutboundProvider
     * 2. Starts Express
     * @params config: IBridgeConfig
     * @returns - None
     **/
    function CoronadoBridge(config) {
        this.config = config;
        this.logger = config.logger;
        this.servers = [];
        // 1. Instantiates the OutboundProvider
        this.outboundProvider = config.outboundProvider;
        // 2. Starts Express
        this.setupExpress();
    }
    /**
     * setupExpress
     * This function creates express servers on the given ports.
     * 1. Create an express server for each port in the config
     * 2. Sets up server route
     * @returns - None
     **/
    CoronadoBridge.prototype.setupExpress = function () {
        var _this = this;
        if (this.logger) {
            this.logger.info("CoronadoBridge - setupExpress - starting setup");
        }
        var ports = this.config.ports || [3000];
        // 1. Create an express server for each port in the config
        ports.forEach(function (port) {
            var app = express_1.default();
            app.use(express_1.default.json());
            app.use(function (req, res, next) {
                res.header('Access-Control-Allow-Origin', '*');
                res.header('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept');
                next();
            });
            // 2. Sets up server route
            app.post('*', function (req, res) {
                if (_this.logger) {
                    _this.logger.info("CoronadoBridge - request received on port:" + port + " -> passing message to outboundProvider");
                }
                // Merge request body/query into one object for the message and add any passed params.
                // Note: Query properties override body properties (example URL: /?exchange=test&topic=coolfactor1)
                var providerReq = { body: req.body, query: req.query };
                if (req.params) {
                    var paramsArray = req.params['0'].split('/');
                    paramsArray.shift(); // The first item is always undefined so lets remove it.
                    providerReq.params = paramsArray;
                }
                _this.outboundProvider
                    .handler(providerReq)
                    .then(function () {
                    if (_this.logger) {
                        _this.logger.info("CoronadoBridge - handler processed message");
                    }
                    res.sendStatus(200);
                })
                    .catch(function (error) {
                    if (error instanceof bridge_error_1.default) {
                        if (_this.logger) {
                            _this.logger.error("CoronadoBridge - handler error " + error.status + " - " + error.message + " ");
                        }
                        res.status(error.status).send({
                            message: error.message,
                        });
                    }
                    else {
                        if (_this.logger) {
                            _this.logger.error("CoronadoBridge - handler error " + error);
                        }
                        res.status(500).send({
                            message: error,
                        });
                    }
                });
            });
            var server = app.listen(port, function () {
                if (_this.logger) {
                    _this.logger.info("CoronadoBridge - started express server on " + port);
                }
            });
            _this.servers.push(server);
        });
        if (this.logger) {
            this.logger.info("CoronadoBridge - setupExpress - setup complete");
        }
    };
    CoronadoBridge.prototype.close = function () {
        var _this = this;
        this.servers.forEach(function (server, index) {
            server.close(function () {
                if (_this.logger) {
                    _this.logger.info("Closed server " + index);
                }
            });
        });
    };
    return CoronadoBridge;
}());
exports.default = CoronadoBridge;
